#include <stdio.h>
#include <stdlib.h>

#include <intrin.h> // for __cpuid, __bittest

#include "cpuid.h"

char SizeOfPhysicalAddressRange();

int main (int argc, char* argv[]) 
{
	int eax;
	CpuId_t regs; // eax, ebx, exc, edx
	int max_eax = 0x10;
	char cpusig[16];
	memset(cpusig, 0, sizeof(cpusig));

	printf("========================\n");
	printf("CPUID from userspace\n");
	printf("========================\n");
	printf("\n");

	for (eax = 0; eax < max_eax; eax++) 
	{
		__cpuid(regs.i, eax);
		printf("CPUID[%.8X]  %#010x %#010x %#010x %#010x\n", \
			eax, regs.eax, regs.ebx, regs.ecx, regs.edx);

		if (0 == eax) {
			memcpy(cpusig, (void*)&(regs.ebx), 4);
			memcpy(cpusig+4, (void*)&(regs.edx), 4);
			memcpy(cpusig+8, (void*)&(regs.ecx), 4);
			printf("CpuSig: %s\n", cpusig);
		}
	}

	printf("\n");
	printf("SizeOfPhysicalAddressRange: %d\n", SizeOfPhysicalAddressRange());

	
	getchar(); // to keep console open
	return 0;
}


char SizeOfPhysicalAddressRange()
{
	int eax;
	CpuId_t regs; // eax, ebx, exc, edx
	char bitres;

	/* first check for 64-bit: CPUID.80000001:EDX[29] = 1
	* See 3.3.1 of Volume 3  (System Programming Guide)
	*/
	eax = 0x80000001;
	__cpuid(regs.i, eax);
	printf("CPUID[%.8X]  %#010x %#010x %#010x %#010x\n", \
		eax, regs.eax, regs.ebx, regs.ecx, regs.edx);
	bitres = _bittest((long*) &regs.edx, 29);
	if (bitres > 0) {

		/* physical address range is implementation-specific and 
		* indicated by CPUID.80000008H:EAX[bits 7-0]
		* Section 3.3.1 of Volume 3 (System Programming Guide)
		*/

		eax = 0x80000008;
		__cpuid(regs.i, eax);
		printf("CPUID[%.8X]  %#010x %#010x %#010x %#010x\n", \
			eax, regs.eax, regs.ebx, regs.ecx, regs.edx);

		return regs.eax & 0xff;
	} else {
		printf("not 64-bit!\n");
	}

	return 0;
}

void PrintCpuBrandString()
{
	int eax;
	CpuId_t regs; // eax, ebx, exc, edx

	/* See Vol. 2A 3-225 for information on brand string
	*/
	eax = 0x80000000;
	__cpuid(regs.i, eax);
	if (!(regs.eax & 0x80000000)) {

	}

}
